% Time Delay Estimation demo using System Objects and Lenovo T440s
%   Copyright 2015 The MathWorks, Inc.
format compact;clc;%clear all;
close all;
imaqreset;

c = 342.26; % Sound speed [m/s] at 20 deg in Celsius
mic_dist = 4.5e-2; % [m]  Microphons distance between left and right
pos_mic = [-mic_dist mic_dist]/2; % [m]
dmax = pos_mic(end)-pos_mic(1)
% Fov_deg = 57; % [deg] Kinect
Fov_deg = 60; % [deg] T440s?
Fov = Fov_deg*pi/180; % [rad] 
Fov_min = -Fov/2; Fov_max =  Fov/2;
umin = sin(Fov_min);
umax = sin(Fov_max);
%% Set AudioRecorder
har = dsp.AudioRecorder('DeviceName','Default'); % ,...C
%      'SampleRate',16000,'NumChannels',2);
har.SampleRate
har.SamplesPerFrame = 1024*4
%% Prepare Sytem Objects
hts = dsp.TimeScope('SampleRate',har.SampleRate, 'YLimits',[-0.2 0.2], 'TimeSpan', 5);
hsa = dsp.SpectrumAnalyzer('SampleRate',har.SampleRate, 'PlotAsTwoSidedSpectrum', false);
hsa.Name = 'Spectrum';
% %pos = hsa.Position; % [left bottom width height] default is  [240 287 800 450] 
max_delay = dmax/c*har.SampleRate
max_lags = ceil(max_delay)+1 % samples
lags = -(max_lags-1):(max_lags-1);
sin_th = sin(lags/max_delay)'; % for plot
%% Set Filter Objects
hFilt = dsp.FIRFilter;
hFilt.Numerator =  [1 -0.98]; % Pre-emphasis filer
%fvtool(hFilt,'Fs',har.SampleRate)
d = fdesign.lowpass('N,Fc',8,6e3,har.SampleRate);
designmethods(d)
hd = design(d,'systemobject',true)
fvtool(hd,'Fs',har.SampleRate)
%% Set Normalizer
hnorm = dsp.Normalizer;
hnorm.Method = '2-norm';
%% Prepare video
rgbViewer  = vision.DeployableVideoPlayer;
% RGB video imput
rgbVid = videoinput('winvideo', 1,'RGB24_640x480');% 'RGB24_1280x720'
pos = rgbVid.ROIPosition;
Xdim = pos(3); Ydim = pos(4);
% Get the VIDEOSOURCE object from it
rgbSrc = getselectedsource(rgbVid);
rgbVid.FramesPerTrigger =  1;
% Initialization of the camera
triggerconfig(rgbVid, 'manual');
start(rgbVid);
%% Prepare Image Padder Objects
xmin = -0.8; xmax = 0.8; ypad = 120; % pixels for correation display area hight
xpad = ceil((Xdim/(umax-umin)*(xmax-xmin)-Xdim)/2) % display width for both sides
himpad = vision.ImagePadder('Method', 'Constant', ...
 		'RowPaddingLocation', 'Bottom', 'NumPaddingRows', ypad, ...
		'ColumnPaddingLocation', 'Both left and right', 'NumPaddingColumns', xpad);
% Set Set Vido player position
scrsz = get(groot,'ScreenSize');
Xcentor = scrsz(3)/2
Xwidth = Xdim+xpad*2
X0 = Xcentor-Xwidth/2+1
rgbViewer.Location = [X0, 440]
% Set Specrum Analyzer position and size
hsa.Position = [X0 50 Xwidth 280];
hsa.ShowLegend = true;
hsa.ChannelNames = {'Mic', 'Filtered'}
%% Prepare Shape, Maker, Text Inserter
shapeInserter_r = vision.ShapeInserter('Shape','Lines',...
    'BorderColor','Custom','CustomBorderColor',uint8([255 0 0]));% [R G B];
shapeInserter_w = vision.ShapeInserter('Shape','Lines',...
    'BorderColor','Custom','CustomBorderColor',uint8([255 255 255]));
X = ceil(sin_th*Xdim/(umax-umin)+Xdim/2+xpad);
markerInserter_w = vision.MarkerInserter('Shape','Star','Size',5,...
    'BorderColor','Custom','CustomBorderColor',uint8([255 255 255]));
textInserter = vision.TextInserter('%d','LocationSource',...
    'Input port','Color',[255, 255, 255],'FontSize',24);
%% Start Demo
tic;
while toc < 120
    Mic = step(har);
    y = step(hFilt,Mic);    % Pre-empasis filter
    y = step(hd,y);         % Low pass filter for T440s
    y1 = y(:,2); y2 = y(:,1);% for T440s
    step(hsa,[Mic(:,2) y2]); % Check specrtrum befor and after filtering
    % step(hts,[y1 y2]);    % You can check wave form
    % Normalized Cross Correlation
%     nxcorr = step(dsp.Crosscorrelator,step(hnorm,y1),step(hnorm,y2));
    nxcorr = xcorr(step(hnorm,y1),step(hnorm,y2),max_lags-1); %'unbiased');
    [yp,ind] = max(nxcorr);
    max_ind = length(nxcorr);
    if ind == 1
        ind = 2;yp = nxcorr(2);
    elseif ind == max_ind
        ind = max_ind-1;yp = nxcorr(ind);
    end;
    % Estimatate Peak position by parabolic interpolation
    lag = ind-max_lags;
    xp = lag-0.5*(nxcorr(ind+1)-nxcorr(ind-1))/(nxcorr(ind+1)+nxcorr(ind-1)-2*yp);
    th_deg = real(asin(xp/max_delay)*180/pi);
    % Show image and peak position line
    rgbFrameData = step(himpad,getsnapshot(rgbVid));
    Xpos = Xdim/2+round(Xdim*xp/max_delay)+xpad;
    rgbFrameData = step(shapeInserter_r,fliplr(rgbFrameData),int32([Xpos 1 Xpos Ydim+ypad]));
    % added fliplr for T440s
    % Peak position maker * and direction in text
    Ypos = Ydim+ypad-(yp*ypad/2+ypad/2);
    rgbFrameData = step(markerInserter_w,rgbFrameData,int32([Xpos Ypos]));
    rgbFrameData = step(textInserter,rgbFrameData,int32(th_deg),int32([Xpos Ypos]));
    % Plot normalized correation function 
    Ycorr =  Ydim+ypad-(nxcorr*ypad/2+ypad/2);
    XY_line = reshape([X Ycorr]',1,length(X)*2);
    rgbFrameData = step(shapeInserter_w,rgbFrameData,int32(XY_line));
    step(rgbViewer, rgbFrameData);
end
stop(rgbVid);release(rgbViewer);
release(har);release(hts);release(hsa);release(hFilt);release(hnorm);
release(himpad);release(shapeInserter_r);release(shapeInserter_w);release(textInserter)